pub type IXMLGraphBuilder = *mut ::core::ffi::c_void;
pub const CLSID_XMLGraphBuilder: ::windows_sys::core::GUID = ::windows_sys::core::GUID { data1: 464542049, data2: 24511, data3: 4562, data4: [165, 33, 68, 223, 7, 193, 0, 0] };
